<?php
/**
 * Medic52 WPStatus setup
 *
 */

defined( 'ABSPATH' ) || exit;

/**
 * Main Medic52_WPStatus Class.
 *
 * @class Medic52_WPStatus
 */
final class Medic52_WPStatus {
    /**
     * Medic52_WPStatus version.
     *
     * @var string
     */
    public $version = '1.0.0';

    /**
     * The single instance of the class.
     *
     * @var Medic52_WPStatus
     * @since 1.0
     */
    protected static $_instance = null;

    /**
     * Main Medic52_WPStatus Instance.
     *
     * Ensures only one instance of Medic52_WPStatus is loaded or can be loaded.
     *
     * @since 1.0
     * @static
     * @return Medic52_WPStatus - Main instance.
     */
    public static function instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * Medic52_WPStatus Constructor.
     */
    public function __construct() {
        $this->define_constants();
        $this->includes();
        $this->init_hooks();
    }

    /**
     * When WP has loaded all plugins, trigger the `medic52_wpstatus_loaded` hook.
     *
     * This ensures `medic52_wpstatus_loaded` is called only after all other plugins
     * are loaded, to avoid issues caused by plugin directory naming changing
     * the load order.
     *
     * @since 1.0.0
     */
    public function on_plugins_loaded() {
        do_action( 'medic52_wpstatus_loaded' );
    }

    /**
     * Hook into actions and filters.
     *
     * @since 1.0
     */
    private function init_hooks() {
        register_activation_hook( MWPS_PLUGIN_BASENAME, array( 'MWPS_Install', 'install' ) );
        register_deactivation_hook(MWPS_PLUGIN_BASENAME, array($this, 'plugin_deactivation'));

        add_action( 'plugins_loaded', array( $this, 'on_plugins_loaded' ), -1 );

        add_action( 'init', array( 'MLTS_Shortcodes', 'init' ) );

        add_action( 'wp_enqueue_scripts', array( __CLASS__, 'load_scripts' ) );

        // header styles
        add_action( 'wp_head', array( $this, 'wp_head' ) );
    }

    /**
     * Define MWPS Constants.
     */
    private function define_constants() {
        $upload_dir = wp_upload_dir( null, false );

        $this->define( 'MWPS_ABSPATH', dirname( MWPS_PLUGIN_FILE ) . '/' );
        $this->define( 'MWPS_PLUGIN_BASENAME', plugin_basename( MWPS_PLUGIN_FILE ) );
        $this->define( 'MWPS_VERSION', $this->version );
        $this->define( 'MWPS_PLUGIN_NAME', 'Medic52 Lift & Trail Status' );
        $this->define( 'MWPS_TEXT_DOMAIN', 'medic52_wpstatus' );

        $ratings = array(
            'easier' => array(
                'label' => 'Easier',
                'color' => '#0d8746',
                'icon' => ''
            ),
            'more_difficult' => array(
                'label' => 'More Difficult',
                'color' => '#2d72c3',
                'icon' => ''
            ),
            'most_difficult' => array(
                'label' => 'Most Difficult',
                'color' => '#000000',
                'icon' => ''
            ),
            'expert' => array(
                'label' => 'Expert',
                'color' => '#000000',
                'icon' => ''
            ),
            'freestyle_terrain' => array(
                'label' => 'Freestyle Terrain',
                'color' => '#e8630b',
                'icon' => ''
            ),
        );
        $this->define( 'MLTS_RATINGS_OPTION', apply_filters('mlts_default_ratings_option', $ratings) );

    }

    /**
     * Define constant if not already set.
     *
     * @param string      $name  Constant name.
     * @param string|bool $value Constant value.
     */
    private function define( $name, $value ) {
        if ( ! defined( $name ) ) {
            define( $name, $value );
        }
    }

    /**
     * Returns true if the request is a non-legacy REST API request.
     *
     * Legacy REST requests should still run some extra code for backwards compatibility.
     *
     * @todo: replace this function once core WP function is available: https://core.trac.wordpress.org/ticket/42061.
     *
     * @return bool
     */
    public function is_rest_api_request() {
        if ( empty( $_SERVER['REQUEST_URI'] ) ) {
            return false;
        }

        $rest_prefix         = trailingslashit( rest_get_url_prefix() );
        $is_rest_api_request = ( false !== strpos( $_SERVER['REQUEST_URI'], $rest_prefix ) ); // phpcs:disable WordPress.Security.ValidatedSanitizedInput.MissingUnslash, WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

        return apply_filters( 'mwps_is_rest_api_request', $is_rest_api_request );
    }

    /**
     * What type of request is this?
     *
     * @param  string $type admin, ajax, cron or frontend.
     * @return bool
     */
    private function is_request( $type ) {
        switch ( $type ) {
            case 'admin':
                return is_admin();
            case 'ajax':
                return defined( 'DOING_AJAX' );
            case 'cron':
                return defined( 'DOING_CRON' );
            case 'frontend':
                return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' ) && ! $this->is_rest_api_request();
        }
    }

    /**
     * Include required core files used in admin and on the frontend.
     */
    public function includes() {

        include_once MWPS_ABSPATH . 'includes/class-mwps-install.php';

        include_once MWPS_ABSPATH . 'includes/class-mwps-shortcodes-lift-trail.php';

        include_once MWPS_ABSPATH . 'includes/mwps-core-functions.php';
        include_once MWPS_ABSPATH . 'includes/class-mwps-shortcodes.php';

        if ( $this->is_request( 'admin' ) ) {
            include_once MWPS_ABSPATH . 'includes/class-mwps-admin.php';
        }
    }
    /**
     * Register/queue frontend scripts.
     */
    public static function load_scripts() {
        wp_enqueue_style(MWPS_TEXT_DOMAIN.'_style', self::get_asset_url( 'assets/css/mwpsstyle.css' ));
        wp_enqueue_script(MWPS_TEXT_DOMAIN.'_script', self::get_asset_url( 'assets/js/mltsscript.js' ), array('jquery'), MWPS_VERSION, true);
    }

    /**
     * Return asset URL.
     *
     * @param string $path Assets path.
     * @return string
     */
    private static function get_asset_url( $path ) {
        return apply_filters( 'mwps_get_asset_url', plugins_url( $path, MWPS_PLUGIN_FILE ), $path );
    }

    /**
     * Get the plugin url.
     *
     * @return string
     */
    public function plugin_url() {
        return untrailingslashit( plugins_url( '/', MWPS_PLUGIN_FILE ) );
    }

    /**
     * Get the plugin path.
     *
     * @return string
     */
    public function plugin_path() {
        return untrailingslashit( plugin_dir_path( MWPS_PLUGIN_FILE ) );
    }

    public function wp_head() {
        $optionsName = MWPS_TEXT_DOMAIN.'_options';
        $options = get_option($optionsName);
        $ratingsOptions = isset($options['ratings']) ? $options['ratings'] : MLTS_RATINGS_OPTION;
        $style = '';
        foreach ($ratingsOptions as $key=>$rating) {
            $color = $rating['color'];
            if(!empty($color)) {
                $style .= '.icon-rating-'.$key.' {color: '.$color.';}';
            }
        }
        echo '<style>'.$style.'</style>';
    }

    public function plugin_deactivation() {
        $optionsName = MWPS_TEXT_DOMAIN.'_options';
        //delete_option( $optionsName );
    }

}